#Copyright 2008-2009  Marius Muja (mariusm@cs.ubc.ca). All rights reserved.
#Copyright 2008-2009  David G. Lowe (lowe@cs.ubc.ca). All rights reserved.
#
#THE BSD LICENSE
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from pyflann.command import BaseCommand
from pyflann.exceptions import FLANNException
from pyflann.io.dataset import read,write

from os.path import isfile
import numpy
import time

from pyflann import compute_ground_truth

class ComputeGroundTruthCommand(BaseCommand):
    
    def __init__(self):
        
        self.parser.add_option("-i", "--input-file", dest="input_file",
                  help="Input dataset file", metavar="FILE")
        self.parser.add_option("-t", "--test-file", dest="test_file",
                  help="Test dataset file (if not present, generated by sampling from the imput dataset", metavar="FILE")
        self.parser.add_option("-m", "--match-file", dest="match_file", default="match.dat",
                  help="Match file to be generated", metavar="FILE")
        self.parser.add_option("-n", "--nn", type="int", default=1,
                  help="How many NN to compute (default 1)")
        self.parser.add_option("-c", "--count", type="int", default=1000,
                  help="How many features to sample in test set if needed. (default: 1000)")

    def execute(self):
        if self.options.input_file==None:
            raise FLANNException("Need an input file")
        print "Reading input data from file "+self.options.input_file
        dataset = read(self.options.input_file)
        
        if self.options.test_file==None:
            raise FLANNException("Need a test file")
        if isfile(self.options.test_file):
            print "Reading test data from file "+self.options.test_file
            testset = read(self.options.test_file)
        else:
            print "Sampling test file"
            testset = sample_dataset(dataset, self.options.count, remove=True)
            dataset = dataset[0:dataset.shape[0]-self.options.count]
            print "Wrinting new dataset file"
            write(dataset,"new_"+self.options.input_file)
            print "Writing testset file"
            write(testset,self.options.test_file)
            
        print "Computing ground truth"
        
        start = time.clock()
        match = compute_ground_truth(dataset, testset, self.options.nn)
        print "It took %g seconds"%(time.clock()-start)
        
        print "Writing match file"
        write(match,self.options.match_file, format="dat")
